@@ -195,6 +195,12 @@ sub _get_ws ( $self, $cb ) {
                 listen_events  => $self->{listen_events},
                 forward_events => $self->{forward_events},
             },
+            on_listen_event => sub ( $ws, $ev ) {    # API server can listen client events
+                return 1;
+            },
+            on_fire_event => sub ( $ws, $ev ) {      # API server can fire client events
+                return 1;
+            },
             on_connect_error => sub ($res) {
                 while ( my $cb = shift $self->{_get_ws_cb}->@* ) {
                     $cb->( undef, $res );
