@@ -7,12 +7,12 @@ use re qw[];
 use Sort::Naturally qw[nsort];
 use PerlIO::Layers qw[];
 
-has color => ( is => 'ro', isa => Bool, default => 1 );    # colorize dump
+has color => ( is => 'ro', isa => Bool, default => 0 );    # colorize dump
 has dump_method => ( is => 'ro', isa => Maybe [Str], default => 'TO_DUMP' );    # dump method for objects, use "undef" to skip call
 has indent => ( is => 'ro', isa => Int, default => 4 );                         # indent spaces
 
-has _indent => ( is => 'lazy', isa => Str, default => sub { my $self = shift; return q[ ] x $self->indent; }, init_arg => undef );
-has _seen => ( is => 'lazy', isa => HashRef, default => sub { {} }, init_arg => undef );
+has _indent => ( is => 'ro', isa => Str, init_arg => undef );
+has _seen => ( is => 'ro', isa => HashRef, default => sub { {} }, init_arg => undef );
 
 our $COLOR = {
     number  => $BOLD . $CYAN,                                                   # numbers
@@ -69,51 +69,63 @@ our $DUMPERS = {
     },
 };
 
-sub run ( $self, @args ) {
-    return $self->_dump(@args);
+sub run ( $self, @ ) {
+    $self->{_indent} = q[ ] x ( $self->{indent} // 4 );
+
+    if ( !$self->{color} ) {
+        return remove_ansi $self->_dump( $_[1] );
+    }
+    else {
+        return $self->_dump( $_[1] );
+    }
 }
 
 # INTERNAL METHODS
 sub _dump ( $self, @ ) {
     my %args = (
-        path    => q[$VAR],
+        path    => '$VAR',
         unbless => 0,
         splice @_, 2,
     );
 
-    local $ENV{ANSI_COLORS_DISABLED} = 1 unless $self->color;
+    local $ENV{ANSI_COLORS_DISABLED} = 1 if !$self->{color};
 
     my ( $var_type, $blessed ) = $self->_var_type( $_[1], unbless => $args{unbless} );
 
     # detect var addr
-    my $var_addr = qq[${var_type}_];
+    my $var_addr = "${var_type}_";
+
     if ( ref $_[1] ) {
-        $var_addr .= refaddr( $_[1] );
+        $var_addr .= refaddr $_[1];
     }
     else {
-        $var_addr .= refaddr( \$_[1] );
+        $var_addr .= refaddr \$_[1];
     }
 
-    my $res;
-    my $tags;
-    if ( $var_addr && exists $self->_seen->{$var_addr} ) {
-        $res = $COLOR->{seen} . $self->_seen->{$var_addr} . $RESET;
+    my ( $res, $tags );
+
+    if ( $var_addr && exists $self->{_seen}->{$var_addr} ) {
+        $res = $COLOR->{seen} . $self->{_seen}->{$var_addr} . $RESET;
     }
     else {
-        $self->_seen->{$var_addr} = $args{path};
+        $self->{_seen}->{$var_addr} = $args{path};
+
         my $dump_method = $blessed ? 'BLESSED' : $var_type;
+
         $dump_method = 'UNKNOWN' if !$self->can($dump_method);
 
         ( $res, $tags ) = $self->$dump_method( $_[1], path => $args{path}, var_type => $var_type );
     }
 
     # weak
-    push @{$tags}, q[weak] if isweak( $_[1] );
+    push $tags->@*, 'weak' if isweak( $_[1] );
+    #
+    # $res .= ',';
 
     # add tags
-    $self->_add_tags( $tags, $res );
+    # $res .= q[ # ] . join q[, ], $tags->@* if $tags;
 
-    return $res;
+    return bless { text => \$res, tags => $tags }, 'Pcore::Core::Dump::Dumper::_Item';
 }
 
 sub _var_type {
@@ -123,9 +135,9 @@ sub _var_type {
         splice( @_, 1 ),
     );
 
-    my $ref_type = reftype( $_[0] );
+    my $ref_type = reftype $_[0];
 
-    if ( my $blessed = blessed( $_[0] ) ) {    # blessed
+    if ( my $blessed = blessed $_[0] ) {    # blessed
         if ( $args{unbless} ) {
             return $ref_type;
         }
@@ -141,7 +153,7 @@ sub _var_type {
     }
     else {
         if ( defined $ref_type ) {
-            if ( $ref_type ~~ [qw[SCALAR VSTRING GLOB]] ) {
+            if ( $ref_type eq 'SCALAR' || $ref_type eq 'VSTRING' || $ref_type eq 'GLOB' ) {
                 return 'REF';
             }
             else {
@@ -149,7 +161,7 @@ sub _var_type {
             }
         }
         else {
-            return CORE::ref( \$_[0] );
+            return CORE::ref \$_[0];
         }
     }
 }
@@ -157,17 +169,7 @@ sub _var_type {
 sub _indent_text {
     my $self = shift;
 
-    my $indent = $self->_indent;
-    $_[0] =~ s/\n/\n$indent/smg;
-
-    return;
-}
-
-sub _add_tags {
-    my $self = shift;
-    my $tags = shift;
-
-    $_[0] .= q[ # ] . join q[, ], $tags->@* if $tags && @{$tags};
+    $_[0] =~ s/\n/\n$self->{_indent}/smg;
 
     return;
 }
@@ -189,13 +191,14 @@ sub _tied_to {
 
     if ($tied) {
         $_[0] //= [];
-        push @{ $_[0] }, 'tied to ' . ref $tied;
+
+        push $_[0]->@*, 'tied to ' . ref $tied;
     }
 
     return;
 }
 
-# REF DUMPERS
+# DUMPERS
 sub UNKNOWN {
     my $self = shift;
     my %args = (
@@ -216,28 +219,26 @@ sub BLESSED {
 
     my $ref = ref $obj;
 
-    my $res = $COLOR->{class} . $ref . ' {' . $RESET . qq[\n];
+    my $res = $COLOR->{class} . $ref . ' {' . $RESET . "\n";
 
-    my $tags;
-
-    my $dumped;
+    my ( $tags, $dumped );
 
     # @ISA
     {
         no strict qw[refs];
 
         if ( my @superclasses = @{ $ref . '::ISA' } ) {
-            $res .= $self->_indent . '@ISA: ' . join q[, ], map { $COLOR->{class} . $_ . $RESET } @superclasses;
+            $res .= $self->{_indent} . '@ISA: ' . join q[, ], map { $COLOR->{class} . $_ . $RESET } @superclasses;
 
-            $res .= qq[,\n];
+            $res .= ",\n";
         }
     }
 
     # reafddr
-    $res .= $self->_indent . 'refaddr: ' . refaddr($obj) . qq[,\n];
+    $res .= $self->{_indent} . 'refaddr: ' . refaddr($obj) . ",\n";
 
     # class dump method
-    if ( my $dump_method = $self->dump_method && $obj->can( $self->dump_method ) ) {
+    if ( my $dump_method = $self->{dump_method} && $obj->can( $self->{dump_method} ) ) {
         my ( $dump, $dump_tags ) = $obj->$dump_method( $self, path => $args{path} );
 
         if ($dump) {
@@ -245,7 +246,7 @@ sub BLESSED {
 
             $self->_indent_text($dump);
 
-            $res .= $self->_indent . $dump;
+            $res .= $self->{_indent} . $dump;
         }
 
         push $tags->@*, $dump_tags->@* if $dump_tags;
@@ -260,7 +261,7 @@ sub BLESSED {
 
             $self->_indent_text($dump);
 
-            $res .= $self->_indent . $dump;
+            $res .= $self->{_indent} . $dump;
         }
 
         push $tags->@*, $dump_tags->@* if $dump_tags;
@@ -273,10 +274,10 @@ sub BLESSED {
 
         $self->_indent_text($blessed);
 
-        $res .= $self->_indent . $blessed;
+        $res .= $self->{_indent} . $blessed;
     }
 
-    $res .= qq[\n] . $COLOR->{class} . '}' . $RESET;
+    $res .= "\n" . $COLOR->{class} . '}' . $RESET;
 
     return $res, $tags;
 }
@@ -289,14 +290,17 @@ sub REF {
         @_,
     );
 
-    return $COLOR->{refs} . q[\\ ] . $RESET . $self->_dump( ${$ref}, path => $args{path} . q[->$*] );
+    my $item = $self->_dump( $ref->$*, path => $args{path} . '->$*' );
+
+    $item->{prefix} = $COLOR->{refs} . '\\ ' . $RESET;
+
+    return "$item";
 }
 
 sub SCALAR {
     my $self = shift;
 
-    my $res;
-    my $tags;
+    my ( $res, $tags );
 
     if ( !defined $_[0] ) {    # undefined value
         $res = $COLOR->{undef} . 'undef' . $RESET;
@@ -310,31 +314,32 @@ sub SCALAR {
         my $length       = length $item;
         escape_scalar( $item, esc_color => $COLOR->{escaped}, reset_color => $COLOR->{string} );
 
-        if ( utf8::is_utf8($item) ) {              # characters
-            push @{$tags}, q[UTF8];
+        if ( utf8::is_utf8 $item ) {               # characters
+            push $tags->@*, 'UTF8';
 
             if ( $bytes_length == $length ) {
-                push @{$tags}, q[single-byte, downgradable];    # ASCII-7bit (bytes in perl terminology), UTF8 flag can be dropped
+                push $tags->@*, 'single-byte, downgradable';    # ASCII-7bit (bytes in perl terminology), UTF8 flag can be dropped
             }
             else {
-                push @{$tags}, q[multi-byte];
+                push $tags->@*, 'multi-byte';
             }
 
-            push @{$tags}, q[len = ] . $length;
+            push $tags->@*, 'chars: ' . $length;
         }
         else {                                                  # octets
             if ( $item =~ /[[:^ascii:]]/sm ) {                  # if has non-ASCII-7bit bytes - treats buffer as binary
-                push @{$tags}, q[latin1];
+                push $tags->@*, 'latin1';
             }
             else {                                              # if contains only ASCII-7bit bytes - treats buffer as string
-                push @{$tags}, q[ASCII];
+                push $tags->@*, 'ASCII';
             }
         }
 
-        push @{$tags}, q[bytes::len = ] . $bytes_length;
-        push @{$tags}, q[tied to ] . ref tied $_[0] if tainted( $_[0] );
+        push $tags->@*, 'bytes: ' . $bytes_length;
 
-        $res = q["] . $COLOR->{string} . $item . $RESET . q["];
+        push $tags->@*, 'tied to ' . ref tied $_[0] if tainted $_[0];
+
+        $res = 'qq[' . $COLOR->{string} . $item . $RESET . ']';
     }
 
     $self->_tied_to( tied $_[0], $tags );
@@ -350,30 +355,37 @@ sub ARRAY {
         @_,
     );
 
-    my $res;
-    my $tags;
-    if ( !@{$array_ref} ) {
-        $res = $COLOR->{refs} . q[[]] . $RESET;
+    my ( $res, $tags );
+
+    if ( !$array_ref->@* ) {
+        $res = $COLOR->{refs} . '[]' . $RESET;
     }
     else {
-        $res = $COLOR->{refs} . q{[} . $RESET . $LF;
+        $res = $COLOR->{refs} . '[' . $RESET . $LF;
+
         my $max_index_length = length( $#{$array_ref} ) + 2;
 
-        for my $i ( 0 .. $#{$array_ref} ) {
-            my $index = sprintf( q[%-*s], $max_index_length, qq[[$i]] ) . q[ ];
-            $res .= $self->_indent . $COLOR->{array} . $index . $RESET;
+        for my $i ( 0 .. $array_ref->$#* ) {
+            my $index = sprintf( '%-*s', $max_index_length, "[$i]" ) . q[ ];
+
+            $res .= $self->{_indent} . $COLOR->{array} . $index . $RESET;
 
             my $el = $self->_dump( $array_ref->[$i], path => $args{path} . "->[$i]" );
+
+            # not last array element
+            if ( $i != $array_ref->$#* ) {
+                $el->{sep} = ',';
+            }
+
             $self->_indent_text($el);
 
-            $res .= $el;
-            $res .= qq[,\n] if $i != $#{$array_ref};    # not last array element
+            $res .= "$el\n";
         }
 
-        $res .= $LF . $COLOR->{refs} . q{]} . $RESET;
+        $res .= $COLOR->{refs} . ']' . $RESET;
     }
 
-    $self->_tied_to( tied @{$array_ref}, $tags );
+    $self->_tied_to( tied $array_ref->@*, $tags );
 
     return $res, $tags;
 }
@@ -386,49 +398,63 @@ sub HASH {
         @_,
     );
 
-    my $res;
-    my $tags;
-    if ( !keys %{$hash_ref} ) {
-        $res = $COLOR->{refs} . q[{}] . $RESET;
+    my ( $res, $tags );
+
+    if ( !keys $hash_ref->%* ) {
+        $res = $COLOR->{refs} . '{}' . $RESET;
     }
     else {
         $res = $COLOR->{refs} . '{' . $RESET . $LF;
+
         my $keys;
         my $max_length = 0;
+
+        # index hash keys
         for ( nsort keys $hash_ref->%* ) {
             my $indexed_key = {
                 raw_key     => $_,
                 escaped_key => \escape_scalar( $_, esc_color => $COLOR->{escaped}, reset_color => $COLOR->{hash} ),
             };
 
+            # hash key requires to be quoted
+            if (/[^[:alnum:]_]/sm) {
+                $indexed_key->{escaped_key} = \( 'q[' . $indexed_key->{escaped_key}->$* . ']' );
+            }
+
             $indexed_key->{escaped_key_nc} = $indexed_key->{escaped_key}->$*;
 
             remove_ansi $indexed_key->{escaped_key_nc};
 
             $indexed_key->{escaped_key_nc_len} = length $indexed_key->{escaped_key_nc};
 
-            push @{$keys}, $indexed_key;
+            push $keys->@*, $indexed_key;
 
             $max_length = $indexed_key->{escaped_key_nc_len} if $indexed_key->{escaped_key_nc_len} > $max_length;
         }
 
         my $indent = $max_length + 8;
 
-        for my $i ( 0 .. $#{$keys} ) {
-            $res .= $self->_indent . q["] . $COLOR->{hash} . $keys->[$i]->{escaped_key}->$* . $RESET . q["];
-            $res .= sprintf q[%*s], ( $max_length - $keys->[$i]->{escaped_key_nc_len} + 4 ), q[ => ];
+        for my $i ( 0 .. $keys->$#* ) {
+            $res .= $self->{_indent} . $COLOR->{hash} . $keys->[$i]->{escaped_key}->$* . $RESET;
+
+            $res .= sprintf '%*s', ( $max_length - $keys->[$i]->{escaped_key_nc_len} + 4 ), ' => ';
 
             my $el = $self->_dump( $hash_ref->{ $keys->[$i]->{raw_key} }, path => $args{path} . '->{"' . $keys->[$i]->{escaped_key_nc} . '"}' );
+
+            # not last hash key
+            if ( $i != $keys->$#* ) {
+                $el->{sep} = ',';
+            }
+
             $self->_indent_text($el);
 
-            $res .= $el;
-            $res .= qq[,\n] if $i != $#{$keys};    # not last hash key
+            $res .= "$el\n";
         }
 
-        $res .= $LF . $COLOR->{refs} . '}' . $RESET;
+        $res .= $COLOR->{refs} . '}' . $RESET;
     }
 
-    $self->_tied_to( tied %{$hash_ref}, $tags );
+    $self->_tied_to( tied $hash_ref->%*, $tags );
 
     return $res, $tags;
 }
@@ -442,9 +468,7 @@ sub VSTRING {
 sub GLOB {
     my $self = shift;
 
-    my $res;
-    my $tags;
-    my $i;
+    my ( $res, $tags, $i );
     my $flags  = [];
     my $layers = q[];
 
@@ -454,29 +478,31 @@ sub GLOB {
 
             $flags = $_->[2];
         }
-        $layers .= qq[:$_->[0]];
+        $layers .= ":$_->[0]";
 
-        $layers .= qq[($_->[1])] if defined $_->[1];    # add layer encoding
+        $layers .= "($_->[1])" if defined $_->[1];    # add layer encoding
 
-        $layers .= q[:utf8] if q[UTF8] ~~ $_->[2];      # add :utf8 layer, if defined
+        $layers .= ':utf8' if 'UTF8' ~~ $_->[2];      # add :utf8 layer, if defined
     }
 
     my $fileno = eval { fileno $_[0] };
 
-    push @{$tags}, nsort @{$flags};
+    push $tags->@*, nsort $flags->@*;
 
-    push @{$tags}, $layers if $layers;
+    push $tags->@*, $layers if $layers;
 
-    push @{$tags}, qq[fileno = $fileno] if defined $fileno;
+    push $tags->@*, "fileno: $fileno" if defined $fileno;
 
     $self->_tied_to( tied $_[0], $tags );
 
     {
         no overloading;
 
-        $res = $COLOR->{glob} . "$_[0]" . $RESET;
+        $res = $COLOR->{glob} . "$_[0]";
     }
 
+    $res .= $RESET;
+
     return $res, $tags;
 }
 
@@ -490,13 +516,14 @@ sub IO {
 sub CODE {
     my $self = shift;
 
-    return $COLOR->{code} . q[sub { ... }] . $RESET;
+    return $COLOR->{code} . 'sub { ... }' . $RESET;
 }
 
 sub REGEXP {
     my $self = shift;
 
     my ( $pat, $flags ) = re::regexp_pattern( $_[0] );
+
     $flags //= q[];
 
     return $COLOR->{regex} . qq[qr/$pat/$flags] . $RESET;
@@ -511,12 +538,25 @@ sub FORMAT {
 sub LVALUE {
     my $self = shift;
 
-    my ( $res, $tags ) = $self->SCALAR( ${ $_[0] } );
-    unshift @{$tags}, 'LVALUE';
+    my ( $res, $tags ) = $self->SCALAR( $_[0]->$* );
+
+    unshift $tags->@*, 'LVALUE';
 
     return $res, $tags;
 }
 
+package Pcore::Core::Dump::Dumper::_Item {
+    use overload    #
+      q[""] => sub {
+        if ( $_[0]->{tags} ) {
+            return ( $_[0]->{prefix} // q[] ) . $_[0]->{text}->$* . ( $_[0]->{sep} // q[] ) . ' # ' . join q[, ], $_[0]->{tags}->@*;
+        }
+        else {
+            return ( $_[0]->{prefix} // q[] ) . $_[0]->{text}->$* . ( $_[0]->{sep} // q[] );
+        }
+      };
+}
+
 1;
 ## -----SOURCE FILTER LOG BEGIN-----
 ##
@@ -524,7 +564,7 @@ sub LVALUE {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    1 | 79, 230, 292         | ValuesAndExpressions::RequireInterpolationOfMetachars - String *may* require interpolation                     |
+## |    1 | 86, 231, 293         | ValuesAndExpressions::RequireInterpolationOfMetachars - String *may* require interpolation                     |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
