@@ -54,9 +54,9 @@ sub run ( $class, $rpc_boot_args ) {
         }
     }
 
-    # create RPC_TERM message listener
+    # create RPC.TERM event listener
     P->listen_events(
-        'RPC_TERM',
+        'RPC.TERM',
         sub ( $event, @ ) {
             $rpc->RPC_ON_TERM if $rpc->can('RPC_ON_TERM');
 
@@ -65,7 +65,7 @@ sub run ( $class, $rpc_boot_args ) {
     );
 
     # compose listen events
-    my $listen_events = ['RPC_TERM'];
+    my $listen_events = ['RPC.TERM'];
 
     {
         no strict qw[refs];
@@ -96,8 +96,8 @@ sub run ( $class, $rpc_boot_args ) {
                             max_message_size => 1_024 * 1_024 * 100,              # 100 Mb
                             pong_interval    => 50,
                             compression      => 0,                                #
-                            on_listen_event  => sub ( $ws, $ev ) { return 1 },    # events currently are enabled
-                            on_fire_event    => sub ( $ws, $ev ) { return 1 },    # events currently are enabled
+                            on_listen_event  => sub ( $ws, $ev ) { return 1 },    # RPC client can listen server events
+                            on_fire_event    => sub ( $ws, $ev ) { return 1 },    # RPC client can fire server events
                             before_connect   => {
                                 listen_events  => $listen_events,
                                 forward_events => ${"$class\::RPC_FORWARD_EVENTS"},
